import { Button } from "@/components/ui/button"
import Process from "./components/Process"
import { useEffect, useState } from "react"
import { ServerSelector } from "./components/ServerSelector";
import { Toaster } from "./components/ui/toaster";
import { useToast } from "./hooks/use-toast";
import { RefreshCcw } from "lucide-react";
import ServerView from "./components/ServerView";
import getToken from "./lib/token";
import { Input } from "./components/ui/input";
import { Pm2ProcessList } from "./types/pm2";

export default function App() {
  const [savedServers, setSavedServers] = useState([]);
  const [processes, setProcesses] = useState([]);
  const [persistentProcesses, setPersistentProcesses] = useState([]);
  const [currentServ, setCurrentServ] = useState("");
  const [loading, setLoading] = useState(false);
  const [selected, setSelected] = useState(null);

  const { toast } = useToast();

  const loadServer = (hideToaster=false) => {
    const token = getToken(localStorage, currentServ);

    if (!currentServ) {
      toast({
        description: "Select a server to get started.",
    })
      return;
    }
    setLoading(true);
    setProcesses([])
    fetch(currentServ + "/processes", { headers: {"access_token" : token || ""}})
    .then(res => {
      if (!res.ok) {
        setLoading(false);
        if (res.status === 401) {
          toast({
            title: "Unauthenticated",
            description: "Maybe you need to supply a token?",
            variant: "destructive"
          })
        } else {
          toast({
            title: "Unexpected error",
            description: "Requested returned status " + res.status,
            variant: "destructive"
          })
        }
      }
      return res.json()
    })
    .then(data => {
      if (data.success) {
        setLoading(false);
        setProcesses(data.processes);
        setPersistentProcesses(data.processes)
        if (hideToaster) return;
        toast({
          title: "Connected to Server",
          description: "Connected to " + currentServ,
      })
      }
    }).catch(err => {
      setLoading(false)
      toast({
        title: "Failed to connect to server",
        description: "Error: " + err,
        variant: "destructive"
    })
    })
  }

  const handleSearch = (e : React.ChangeEvent<HTMLInputElement>) => {
    const value = e.target.value;
    const newFilter : any = [];
    if (!value || value === "") {
      setProcesses(persistentProcesses);
    } else {
      processes.forEach((r : Pm2ProcessList) => {
        const isVisible = r.name.toLowerCase().includes(value.toLowerCase())
        if (isVisible) {
          newFilter.push(r)
        }
      })
      setProcesses(newFilter)
    }

    if (e.target.value === "") {
      setProcesses(persistentProcesses);
    } else {
      return;
      // const filtered = persistentProcesses.filter(p => p.name
    }
  }

  useEffect(() => {
    loadServer();
  }, [currentServ])

  return (
    <div className="container mx-auto py-10">
      {
        selected ? (
          <ServerView currentServer={currentServ} selected={selected} setSelected={setSelected} />
        ) : (
          <>
            <div className="flex flex-row justify-between">
              <div>
                <ServerSelector savedServers={savedServers} setSavedServers={setSavedServers} currentServ={currentServ} setCurrentServ={setCurrentServ}/>
                <Button className="ml-2" onClick={() => { loadServer() }}><RefreshCcw /></Button>
              </div>
              <Input onChange={handleSearch} placeholder='Search...' className="w-52"/>
            </div>
            <Process currentServer={currentServ} processes={processes} loading={loading} loadServer={loadServer} setSelected={setSelected} />
          </>
        )
      }
      <Toaster />
    </div>
  )
}