import { useToast } from '@/hooks/use-toast';
import React from 'react'
import { Button } from './ui/button';
import { Cpu, Hash, HomeIcon, MemoryStick, Octagon, Play, RefreshCcw, RotateCcw } from 'lucide-react';
import {
    Table,
    TableBody,
    TableCell,
    TableHead,
    TableHeader,
    TableRow,
} from "@/components/ui/table"

import sendSignal from '@/lib/pm2';
import Status from './Status';
import getToken from '@/lib/token';
import { Pm2ProcessList } from '@/types/pm2';

export default function ServerView({ currentServer, selected, setSelected } : { currentServer: string, selected: string, setSelected: React.SetStateAction<any> }) {
    const [logs, setLogs] = React.useState([]);
    const [errLogs, setErrLogs] = React.useState([]);
    const [currData, setCurrData] = React.useState<Pm2ProcessList>();

    const { toast } = useToast();

    const loadInfo = () => {
        if (!selected) {
            toast({
            description: "Not Server Selected.",
            variant: "destructive"
            })
            return;
        }
        setLogs([]);
        setErrLogs([])

        const token = getToken(localStorage, currentServer);

        fetch(currentServer + `/${selected}/info`, {method: "GET", headers: {"access_token": token}})
        .then(res => {
            if (!res.ok) {
                if (res.status === 401) {
                toast({
                    title: "Unauthenticated",
                    description: "Maybe you need to supply a token?",
                    variant: "destructive"
                })
                } else {
                toast({
                    title: "Unexpected error",
                    description: "Requested returned status " + res.status,
                    variant: "destructive"
                })
                }
            }
            return res.json()
        })
        .then(data => {
            console.log(data);
            if (data.success) {
                setCurrData(data.processes[0]);
            }
        }).catch(err => {
            toast({
            title: "Failed to fetch info",
            description: "Error: " + err,
            variant: "destructive"
        })
        })
    }

    const loadLogs = () => {
        if (!selected) {
            toast({
            description: "Not Server Selected.",
            variant: "destructive"
            })
            return;
        }
        const token = getToken(localStorage, currentServer)
        setLogs([]);
        setErrLogs([])
        fetch(currentServer + `/${selected.replace(" ", "-")}/logs`, {method: "POST", headers: {"access_token": token}})
        .then(res => {
            if (!res.ok) {
                if (res.status === 401) {
                    toast({
                        title: "Unauthenticated",
                        description: "Maybe you need to supply a token?",
                        variant: "destructive"
                    })
                } else {
                toast({
                    title: "Unexpected error",
                    description: "Requested returned status " + res.status,
                    variant: "destructive"
                })
                }
            }
            return res.json()
        })
        .then(data => {
            console.log(data);
            if (data.success) {
                setLogs(data.logs)
                setErrLogs(data.error)
            }
        }).catch(err => {
            toast({
            title: "Failed to fetch logs",
            description: "Error: " + err,
            variant: "destructive"
        })
        })
    }

    React.useEffect(() => {
        loadLogs();
        loadInfo();
    }, [selected])

    const bytesToMB = (bytes : number) => {
        return (bytes / (1024 * 1024)).toFixed(2);
    }

    return (
        <div>
            <div className="flex flex-row justify-between mb-4">
                <div className="">
                    <h1 className='text-5xl'>{selected}</h1>
                    <div className="flex flex-row mt-2">
                        <Status status={currData?.pm2_env.status ?? "Unknown"} />
                        <Hash  className="ml-2 text-gray-700" />
                        <p className='ml-1'>{currData?.pm2_env.pm_id}</p>
                        <Cpu className="ml-2 text-gray-700" />
                        <p className='ml-1'>{bytesToMB(currData?.monit.cpu ?? 0)}%</p>
                        <MemoryStick className="ml-2 text-gray-700" />
                        <p className='ml-1'>{bytesToMB(currData?.monit.memory ?? 0)}mb</p>
                        <RotateCcw className="ml-2 text-gray-700" />
                        <p className='ml-1'>{currData?.pm2_env.restart_time}</p>
                    </div>
                </div>

                <div className="mt-2">
                    <Button variant="secondary" onClick={() => setSelected(null)}><HomeIcon /></Button>
                    <Button variant="secondary" className="ml-1" onClick={() => { loadLogs();loadInfo(); }}><RefreshCcw /></Button>
                    <Button variant="secondary" className="ml-1" onClick={() => {sendSignal(currentServer, selected, "resume", loadInfo)}}><Play /></Button>
                    <Button variant="secondary" className="ml-1" onClick={() => {sendSignal(currentServer, selected, "stop", loadInfo)}}><Octagon /></Button>
                    <Button variant="secondary" className="ml-1" onClick={() => {sendSignal(currentServer, selected, "restart", loadInfo)}}><RotateCcw /></Button>
                </div>
            </div>

            <p className="text-3xl mb-3">Process Information</p>
            <Table className='mb-3'>
            <TableHeader>
                <TableRow>
                <TableHead>Description</TableHead>
                <TableHead>Value</TableHead>
                </TableRow>
            </TableHeader>
            <TableBody>
                <TableRow>
                    <TableCell>Version</TableCell>
                    <TableCell>{currData?.pm2_env.version}</TableCell>
                </TableRow>
                <TableRow>
                    <TableCell>Created At</TableCell>
                    <TableCell>{new Date(currData?.pm2_env.created_at ?? 0).toUTCString()}</TableCell>
                </TableRow>
                <TableRow>
                    <TableCell>Last Exit Code</TableCell>
                    <TableCell>{currData?.pm2_env.exit_code}</TableCell>
                </TableRow>
                <TableRow>
                    <TableCell>Unstable Restarts</TableCell>
                    <TableCell>{currData?.pm2_env.unstable_restarts}</TableCell>
                </TableRow>
            </TableBody>
            </Table>

            <p className='text-gray-600 mb-1'>Standard Output</p>
            <div className='bg-black text-white w-auto h-[250px] overflow-auto p-4 rounded'>
                <code>
                    {logs.length === 1 && logs[0] === "" ? (
                        <p>No Content</p>
                    ) : null}
                    {logs.map((line, index) => (
                        <p key={index}>{line}</p>
                    ))}
                </code>
            </div>
            <hr/>
            <p className='text-gray-600 mt-3 mb-1'>Standard Error</p>
            <div className='bg-black text-red-500 w-auto h-[250px] overflow-auto p-4 rounded'>
                <code>
                    {errLogs.length === 1 && errLogs[0] === "" ? (
                        <p>No Content</p>
                    ) : null}
                    {errLogs.map((line, index) => (
                        <p key={index}>{line}</p>
                    ))}
                </code>
            </div>
            
            <details className='mt-3'>
                <summary>View Raw Information</summary>
                <div className='bg-gray-300 w-auto h-[400px] overflow-auto p-4 rounded mt-2'>
                    <pre>
                        {JSON.stringify(currData, null, 2)}
                    </pre>
                </div>
            </details>

        </div>
    )
}
